import cv2
import numpy as np
import halcon
from our1314.myutils.myutils import rad, contours2coord
from halcon.numpy_interop import himage_as_numpy_array, himage_from_numpy_array

class templateMatch():
    def __init__(self, src, coord):
        
        c1,r1,c2,r2 = coord
        model_region = halcon.gen_rectangle1(int(r1),int(c1),int(r2),int(c2))
        temp = halcon.reduce_domain(src, region=model_region)
        self.model_id = halcon.create_scaled_shape_model(temp, 5, -rad(10.0), rad(20.0), rad(0.3657), 0.8, 1/0.8, 0.01, ['none', 'no_pregeneration'], 'use_polarity', [20,50,30], 10)
        
        _, r1, c1 = halcon.area_center(model_region)
        r2, c2, phi, len1, len2 = halcon.smallest_rectangle2(model_region)

        halcon.set_shape_model_origin(self.model_id, r2[0] - r1[0], c2[0] - c1[0])
        self.select_rect = halcon.get_shape_model_contours(self.model_id, 1)
        select_pts = contours2coord(self.select_rect).T

        dis = himage_as_numpy_array(src)
        pt1 = select_pts[0]
        for pt2 in select_pts[1:]:
            cv2.line(dis, pt1, pt2, (0,0,255), 1)
            pt1 = pt2
        cv2.imshow("dis", dis)
        cv2.waitKey()
        pass

    def create(self):

        pass

    def createmodel(self):
        
        pass


    def aa(self):
        halcon.find_scaled_shape_model()
        pass


if __name__ == '__main__':
    from our1314.cv.mouseselect import mouseSelect

    path = 'd:/desktop/1.jpg'
    src = cv2.imdecode(np.fromfile(path, dtype=np.uint8), cv2.IMREAD_COLOR)#type:np.ndarray
    a = mouseSelect(src)

    coord = a.pt1[0], a.pt1[1], a.pt2[0], a.pt2[1]
    src = himage_from_numpy_array(src)
    match = templateMatch(src, coord)


    


import torchvision
import torch
import numpy as np
import cv2
import torchvision.transforms.functional as F
import random
from PIL import Image

# 按比例将长边缩放至目标尺寸
class Resize1:
    def __init__(self, width):
        self.width = width

    def __call__(self, x):
        if isinstance(torch.Tensor):
            _, h, w = x.shape
            scale = self.width / max(w, h)
            W, H = round(w * scale), round(h * scale)
            x = F.resize(x, [H,W])
            return x
        elif isinstance(np.ndarray):
            h, w, c = x.shape
            scale = self.width / max(w, h)
            W, H = round(scale * w), round(scale * h)
            x = cv2.resize(x, (W, H), interpolation=cv2.INTER_LINEAR)
            return x

class PadSquare:
    def __call__(self, x):
        if isinstance(torch.Tensor):
            _, h, w = x.shape
            width = max(w, h)
            pad_left = round((width - w) / 2.0)
            pad_right = width - w - pad_left
            pad_up = round((width - h) / 2.0)
            pad_down = width - h - pad_up

            x = F.pad(x, [pad_left, pad_up, pad_right, pad_down])
            return x

        elif isinstance(np.ndarray):
            h, w, _ = x.shape
            width = max(w, h)
            pad_left = round((width - w) / 2.0)
            pad_right = width - w - pad_left
            pad_up = round((width - h) / 2.0)
            pad_down = width - h - pad_up

            x = cv2.copyMakeBorder(x, pad_up, pad_down, pad_left, pad_right, cv2.BORDER_CONSTANT, value=0)
            return x

class randomaffine_imgs:
    def __init__(self, rotate:list[float], transx:list[float], transy:list[float], scale:list[float]):
        self.rot_deg = random.uniform(rotate[0], rotate[1])
        self.transx = random.uniform(transx[0], transx[1])
        self.transy = random.uniform(transy[0], transy[1])
        self.scale = random.uniform(min(scale), max(scale))

    def __call__(self, imgs:list[torch.Tensor]):
        result = []
        for x in imgs:
            _,h,w = x.shape
            x = F.affine(x, self.rot_deg, [int(self.transx*w),int(self.transy*h)], self.scale,1, interpolation=F.InterpolationMode.NEAREST)
            result.append(x)
        return result

class randomvflip_imgs:
    def __init__(self, p=0.5):
        self.p = p

    def __call__(self, imgs:list[torch.Tensor]):
        value = random.uniform(0,1)
        if value < self.p:
            imgs = [F.vflip(x) for x in imgs]
        return imgs
    
class randomhflip_imgs:
    def __init__(self, p=0.5):
        self.p = p

    def __call__(self, imgs:list[torch.Tensor]):
        value = random.uniform(0,1)
        if value < self.p:
            imgs = [F.hflip(x) for x in imgs]
        return imgs

# if __name__ == "__main__":
#     a = randomaffine_imgs([-10,10],[-0.1,0.1],[-0.1,0.1],[0.9,1/0.9])
#     image = cv2.imdecode(np.fromfile('D:/desktop/choujianji/roi/mask/LA22089071-0152_2( 4, 17 ).jpg', dtype=np.uint8), cv2.IMREAD_UNCHANGED) # type:cv2.Mat
#     label = cv2.imdecode(np.fromfile('D:/desktop/choujianji/roi/mask/LA22089071-0152_2( 4, 17 ).png', dtype=np.uint8), cv2.IMREAD_UNCHANGED) # type:cv2.Mat
#     image = F.to_tensor(image)
#     label = F.to_tensor(label)
#     b1,b2 = a([image, label])
#     F.to_pil_image(b1).show()
#     F.to_pil_image(b2).show()



import numpy as np
from math import *

def SE2(x,y,rad):
    r = np.array([
        [cos(rad),-sin(rad),x],
        [sin(rad),cos(rad),y],
        [0,0,1]
        ])
    return r

def _SE2(H):
    x,y = H[0,2],H[1,2]
    cos_theta = H[0,0]
    sin_theta = H[1,0]
    theta = atan2(cos_theta, sin_theta)
    return x,y,theta

#绕3D坐标系X轴旋转的旋转矩阵
def RX(rad):
    r = np.array([
        [1,0,0],
        [0,cos(rad),-sin(rad)],
        [0,sin(rad),cos(rad)]
        ])
    return r
#绕3D坐标系X轴旋转的旋转矩阵
def RY(rad):
    r = np.array([
        [cos(rad),0,sin(rad)],
        [0,1,0],
        [-sin(rad),0,cos(rad)]
        ])
    return r
#绕3D坐标系X轴旋转的旋转矩阵
def RZ(rad):
    r = np.array([
        [cos(rad),-sin(rad),0],
        [sin(rad),cos(rad),0],
        [0,0,1]
        ])
    return r

def Pxyz(x,y,z):
    H = np.eye(4)
    H[0,3],H[1,3],H[2,3]=x,y,z
    return H

def Homogeneous(m):
    h,w = m.shape
    m = np.column_stack([m,np.zeros(h,1)])
    m = np.row_stack([m,np.zeros(1,w+1)])
    m[-1,-1]=1
    return m

def SE3(px,py,pz,rx,ry,rz):
    Rx = Homogeneous(RX(rx))
    Ry = Homogeneous(RY(ry))
    Rz = Homogeneous(RZ(rz))
    P = Pxyz(px,py,pz)
    H = P@Rz@Ry@Rx
    return H

def _SE3(H):
    pass


def deg(rad):
    return rad*180/pi

def rad(deg):
    return deg*pi/180


import cv2
import numpy as np

class mouseSelect_simple():
    def __init__(self, src, windowName='dis'):
        self.src = src
        self.windowName = windowName
        self.down = False
        
        cv2.namedWindow(windowName)
        cv2.setMouseCallback(windowName, self.onmouse)
        cv2.imshow(windowName, src)
        cv2.waitKey()

    def onmouse(self, *p):
        event, x, y, flags, param = p   
        if event == cv2.EVENT_LBUTTONDOWN:
            self.down = True
            self.pt1 = np.array([x,y])

        if event == cv2.EVENT_MOUSEMOVE and self.down==True:
            self.pt2 = np.array([x,y])
            dis = self.src.copy()
            cv2.rectangle(dis, self.pt1, self.pt2, (0,0,255), 2)
            cv2.imshow(self.windowName , dis)

        if event == cv2.EVENT_LBUTTONUP:
            self.down = False
            self.pt2 = np.array([x,y])
            return self.pt1, self.pt2
        
        if event == cv2.EVENT_RBUTTONDOWN:
            cv2.waitKey(200)
            cv2.destroyWindow(self.windowName)


# if __name__ == '__main__':
#     path = input('输入图像路径：')
#     src = cv2.imdecode(np.fromfile(path, dtype=np.uint8), cv2.IMREAD_COLOR)#type:np.ndarray
#     a = mouseSelect(src)
#     print(f'{a.pt1},{a.pt2}')
    


from setuptools import setup, find_packages

setup(
    name='our1314',
    version='0.1.21',
    keywords='our1314',
    description='a library for work',
    license='MIT License',
    url='https://github.com/our1314/our1314-python',
    author='pp',
    author_email='our1314@foxmail.com',
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    install_requires=[],
)
